//	em7slib		Electronicmagnetic 7-segment module driver
//wt]xH쎺	@jsdiy	http://jsdiy.starfree.jp
//	2021/09

/*
74HC595								  ->: E	:->	TPIC6B595-1	->	TPIC6B595-2
QH	dig4-set	->	dig4-segA_G-set		: M	:	DR7	x			DR7	x
QG	dig4-reset	->	dig4-segA_G-reset	: 7	:	DR6	segG-reset	DR6	segG-set
QF	dig3-set	->	dig3-segA_G-set		: S	:	DR5	segF-reset	DR5	segF-set
QE	dig3-reset	->	dig3-segA_G-reset	: e	:	DR4	segE-reset	DR4	segE-set
QD	dig2-set	->	dig2-segA_G-set		: g	:	DR3	segD-reset	DR3	segD-set
QC	dig2-reset	->	dig2-segA_G-reset	: 	:	DR2	segC-reset	DR2	segC-set
QB	dig1-set	->	dig1-segA_G-set		: 4	:	DR1	segB-reset	DR1	segB-set
QA	dig1-reset	->	dig1-segA_G-reset	:dig:	DR0	segA-reset	DR0	segA-set

						TPIC6B595-2	 :	TPIC6B595-1
 <-	SR-OUT					  SR-IN	 <-	SR-OUT					  SR-IN <-
	DR7	DR6	DR5	DR4	DR3	DR2	DR1	DR0	 :	DR7	DR6	DR5	DR4	DR3	DR2	DR1	DR0	
	x	Gs	Fs	Es	Ds	Cs	Bs	As	 :	x	Gr	Fr	Er	Dr	Cr	Br	Ar
*/

#include <avr/io.h>
#include <util/delay.h>
#include "em7slib.h"


//EU74HC595̃f[^V[g^C~O`[gASĂ̑҂Ԃ̂Œ̂̂100ns.
//@VCC=2VɂVAf[^õVtgWX^NbNグ܂ł̎ԁB
//#define	nop()	asm volatile("nop"::)
//#define	wait()	{ nop(); nop(); }

//֐
static	void	SetHi(uint8_t bv);
static	void	SetLo(uint8_t bv);
//
static	void	BeginDigit(void);
static	void	EndDigit(void);
static	void	BeginSegment(void);
static	void	EndSegment(void);
//
static	void	RegisterClear(void);
static	void	SerialClock(void);
static	void	SerialIn(uint8_t bitData);
static	void	PallarelOut(void);
static	void	SegmentFlip(uint8_t segmentPattern, uint8_t flipBitValue);

//ϐ
static	uint8_t	bvSCK, bvRCK;


//
void	EM7S_Init(void)
{
	//|[gݒ肷
	EM7S_DDR	|=	(EM7S_DIG_SCK | EM7S_SEG_SCK | EM7S_SRIN | EM7S_DIG_RCK | EM7S_SEG_RCK | EM7S_SRCLR);	//o͕ɐݒ肷
	SetLo(EM7S_DIG_SCK | EM7S_SEG_SCK | EM7S_SRIN | EM7S_DIG_RCK | EM7S_SEG_RCK);	//Loo͂F_̐MȂ̂
	SetHi(EM7S_SRCLR);	//Hio͂F_̐MȂ̂
}

//VtgWX^̏o͂ҋ@Ԃɂ
//EƃZOg̑So͂LoɂB
void	EM7S_Standby(void)
{
	SetLo(EM7S_SRIN);	//VA͂̒l'0'ɂ
	RegisterClear();	//ƃZOg̗̃VtgWX^NA
	
	BeginDigit();
	PallarelOut();
	EndDigit();
	
	BeginSegment();
	PallarelOut();
	EndSegment();
}

//̃|[gHiɃZbg
static	void	SetHi(uint8_t bv)
{
	EM7S_PORT |= bv;
}

//̃|[gLoɃZbg
static	void	SetLo(uint8_t bv)
{
	EM7S_PORT &= ~bv;
}

//Jn
static	void	BeginDigit(void)
{
	bvSCK = EM7S_DIG_SCK;
	bvRCK = EM7S_DIG_RCK;
}
//
static	void	EndDigit(void)
{
	//Ȃ
}

//ZOgJn
static	void	BeginSegment(void)
{
	bvSCK = EM7S_SEG_SCK;
	bvRCK = EM7S_SEG_RCK;
}
//
static	void	EndSegment(void)
{
	//Ȃ
}

//VtgWX^SLoɂ
//ESRCLR͕_  񓯊iNbNsvjB
static	void	RegisterClear(void)
{
	SetLo(EM7S_SRCLR);	//_
	SetHi(EM7S_SRCLR);
}

//VANbN1NbN
static	void	SerialClock(void)
{
	SetHi(bvSCK);
	SetLo(bvSCK);
}

//VtgWX^̓epo͂
static	void	PallarelOut(void)
{
	SetHi(bvRCK);
	SetLo(bvRCK);
}

//f[^1rbgVA͂
//	bitData: 1 or 0.
static	void	SerialIn(uint8_t bitData)
{
	if (bitData)
	{
		SetHi(EM7S_SRIN);
	}
	else
	{
		SetLo(EM7S_SRIN);
	}
	
	SerialClock();
}

//ZOg̕\XV
//	digit:	1,2,3,4i1,2,3,4ځj
//		segmentPattern:	Seg_xx
//EVtgWX^1͂ÂƉx0͂A擪1ړĨrbgʒu܂ňړB
//EVtgWX^̓[NAĂOB
void	EM7S_Update(uint8_t digit, uint8_t segmentPattern)
{
	//Resetw肷
	BeginDigit();
	
	SerialIn(1);	//1-Reset̃rbgʒu1Zbg
	for (uint8_t i = 0; i < digit - 1; i++)
	{
		SerialIn(0);	//1,2,3-Set̃rbgʒu܂Ő擪1ړ
		SerialIn(0);	//2,3,4-Reset̃rbgʒu܂Ő擪1ړ
	}
	PallarelOut();
	EndDigit();
	
	//ZOgp^[ɏ]ăZOgReset
	BeginSegment();
	
	SerialIn(1);	//VtgWX^1iڂ̃ZOgA-Reset̃rbgʒu1Zbg
	SegmentFlip(segmentPattern, 0);		//ZOgA-GɂReset삷
	EndSegment();
	
	//Setw肷
	BeginDigit();
	SerialIn(0);	//n-Reset̃rbgʒu猅n-Set̃rbgʒu֐擪1ړ
	PallarelOut();
	EndDigit();
	
	//ZOgp^[ɏ]ăZOgSet
	BeginSegment();
	SerialIn(0);	//VtgWX^2iڂ̃ZOgA-Set̃rbgʒuɐ擪1ړ
	SegmentFlip(segmentPattern, 1);		//ZOgA-GɂSet삷
	EndSegment();
	
	//EZOg̃VtgWX^NAASĂ̏o͂Loɂ
	//ESẴRCɓdĂȂƂmɂ邽߁iRCĂh~jB
	EM7S_Standby();
}

//ZOg𔽓]
//	segmentPattern:	Seg_xx
//		flipBitValue:	0:	segmentPatterñrbg0̃ZOg𔽓](Reset)
//						1:	segmentPatterñrbg1̃ZOg𔽓](Set)
static	void	SegmentFlip(uint8_t segmentPattern, uint8_t flipBitValue)
{
	//ZOgA-Gi7jɂđ삷
	//E7ZOg̑삪IƂ擪1̓VtgWX^8rbg(QH)ɂB
	for (uint8_t i = 0; i < 7; i++)
	{
		uint8_t bitData = ((segmentPattern >> i) & 0x01);
		if (bitData == flipBitValue)
		{
			PallarelOut();
			//_delay_ms(15);	//ZOgIɔ]I܂ň莞ԃz[hiMagE7Segf[^V[g15msj
			_delay_ms(5);	//7ZOg̔]̗ꂪX[YɌ悤ZԂɂi5msł蓮삵Ăj
		}
		SerialIn(0);	//̃ZOg̃rbgʒu֐擪1ړ
	}
}
